"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const textDocumentEvents_1 = require("../../util/textDocumentEvents");
const elmAnalyseDiagnostics_1 = require("./elmAnalyseDiagnostics");
const elmMakeDiagnostics_1 = require("./elmMakeDiagnostics");
class DiagnosticsProvider {
    constructor(connection, elmWorkspaceFolder, documentEvents, settings) {
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.getDiagnostics = this.getDiagnostics.bind(this);
        this.newElmAnalyseDiagnostics = this.newElmAnalyseDiagnostics.bind(this);
        this.elmMakeIssueToDiagnostic = this.elmMakeIssueToDiagnostic.bind(this);
        this.events = new textDocumentEvents_1.TextDocumentEvents(documentEvents);
        this.connection = connection;
        this.settings = settings;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.elmMakeDiagnostics = new elmMakeDiagnostics_1.ElmMakeDiagnostics(connection, elmWorkspaceFolder, settings);
        this.elmAnalyseDiagnostics = new elmAnalyseDiagnostics_1.ElmAnalyseDiagnostics(connection, elmWorkspaceFolder, this.newElmAnalyseDiagnostics);
        this.currentDiagnostics = { elmMake: new Map(), elmAnalyse: new Map() };
        this.events.on("open", this.getDiagnostics);
        this.events.on("change", this.getDiagnostics);
        this.events.on("save", this.getDiagnostics);
    }
    newElmAnalyseDiagnostics(diagnostics) {
        this.currentDiagnostics.elmAnalyse = diagnostics;
        this.sendDiagnostics();
    }
    sendDiagnostics() {
        const allDiagnostics = new Map();
        for (const [uri, diagnostics] of this.currentDiagnostics.elmAnalyse) {
            allDiagnostics.set(uri, diagnostics);
        }
        for (const [uri, diagnostics] of this.currentDiagnostics.elmMake) {
            allDiagnostics.set(uri, (allDiagnostics.get(uri) || []).concat(diagnostics));
        }
        for (const [uri, diagnostics] of allDiagnostics) {
            this.connection.sendDiagnostics({ uri, diagnostics });
        }
    }
    getDiagnostics(document) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = vscode_uri_1.URI.parse(document.uri);
            const text = document.getText();
            this.elmAnalyseDiagnostics.updateFile(uri, text);
            this.currentDiagnostics.elmMake = yield this.elmMakeDiagnostics.createDiagnostics(uri);
            this.sendDiagnostics();
        });
    }
    elmMakeIssueToDiagnostic(issue) {
        const lineRange = vscode_languageserver_1.Range.create(issue.region.start.line === 0
            ? issue.region.start.line
            : issue.region.start.line - 1, issue.region.start.column === 0
            ? issue.region.start.column
            : issue.region.start.column - 1, issue.region.end.line === 0
            ? issue.region.end.line
            : issue.region.end.line - 1, issue.region.end.column === 0
            ? issue.region.end.column
            : issue.region.end.column - 1);
        return vscode_languageserver_1.Diagnostic.create(lineRange, issue.overview + " - " + issue.details.replace(/\[\d+m/g, ""), this.severityStringToDiagnosticSeverity(issue.type), undefined, "Elm");
    }
    severityStringToDiagnosticSeverity(severity) {
        switch (severity) {
            case "error":
                return vscode_languageserver_1.DiagnosticSeverity.Error;
            case "warning":
                return vscode_languageserver_1.DiagnosticSeverity.Warning;
            default:
                return vscode_languageserver_1.DiagnosticSeverity.Error;
        }
    }
}
exports.DiagnosticsProvider = DiagnosticsProvider;
//# sourceMappingURL=diagnosticsProvider.js.map