"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const treeUtils_1 = require("./treeUtils");
class HintHelper {
    static createHint(node) {
        if (node) {
            if (node.type === "module_declaration") {
                return this.createHintFromModule(node);
            }
            else {
                return this.createHintFromDefinition(node);
            }
        }
    }
    static createHintFromFunctionParameter(node) {
        if (node &&
            node.parent &&
            node.parent.parent &&
            node.parent.parent.previousNamedSibling &&
            node.parent.parent.previousNamedSibling.type === "type_annotation" &&
            node.parent.parent.previousNamedSibling.lastNamedChild) {
            const functionParametrNodes = treeUtils_1.TreeUtils.findAllNamedChildsOfType(["pattern", "lower_pattern"], node.parent);
            if (functionParametrNodes) {
                const matchIndex = functionParametrNodes.findIndex(a => a === node);
                const typeAnnotationNodes = treeUtils_1.TreeUtils.findAllNamedChildsOfType(["type_ref", "type_expression"], node.parent.parent.previousNamedSibling.lastNamedChild);
                if (typeAnnotationNodes) {
                    const annotation = typeAnnotationNodes[matchIndex];
                    return this.formatHint(annotation ? annotation.text : "", "Local parameter");
                }
            }
        }
        return "Local parameter";
    }
    static createHintFromDefinition(declaration) {
        if (declaration) {
            let comment = "";
            let annotation = "";
            if (declaration.previousNamedSibling) {
                if (declaration.previousNamedSibling.type === "type_annotation") {
                    annotation = declaration.previousNamedSibling.text;
                    if (declaration.previousNamedSibling.previousNamedSibling &&
                        declaration.previousNamedSibling.previousNamedSibling.type ===
                            "block_comment") {
                        comment =
                            declaration.previousNamedSibling.previousNamedSibling.text;
                    }
                }
                else if (declaration.previousNamedSibling.type === "block_comment") {
                    comment = declaration.previousNamedSibling.text;
                }
            }
            return this.formatHint(annotation, comment);
        }
    }
    static createHintFromModule(moduleNode) {
        if (moduleNode) {
            let comment = "";
            if (moduleNode.nextNamedSibling &&
                moduleNode.nextNamedSibling.type === "block_comment") {
                comment = moduleNode.nextNamedSibling.text;
            }
            return this.formatHint("", comment);
        }
    }
    static formatHint(annotation, comment) {
        let value = "";
        if (annotation) {
            value += this.wrapCodeInMarkdown(annotation);
        }
        if (comment) {
            if (value.length > 0) {
                value += "\n\n---\n\n";
            }
            value += this.stripComment(comment);
        }
        return value;
    }
    static stripComment(comment) {
        let newComment = comment;
        if (newComment.startsWith("{-|")) {
            newComment = newComment.slice(3);
        }
        if (newComment.startsWith("{-")) {
            newComment = newComment.slice(2);
        }
        if (newComment.endsWith("-}")) {
            newComment = newComment.slice(0, -2);
        }
        return newComment.trim();
    }
    static wrapCodeInMarkdown(code) {
        return `\n\`\`\`elm\n${code}\n\`\`\`\n`;
    }
}
exports.HintHelper = HintHelper;
//# sourceMappingURL=hintHelper.js.map